"""add_device.py
"""
from fillet.core import full_binaries_path, minknow_config_path
import subprocess


def run(parser, args):
    p = full_binaries_path("config_editor")
    base_cmd = "{} --conf application --filename {} --set".format(
        p, minknow_config_path("app_conf")
    )
    if args.remove:
        """
        config_editor --conf application --filename ../conf/app_conf --set_default data_generation.simulated_device
        config_editor --conf application --filename ../conf/app_conf --set_default device.simulator_device_count
        """
        cmd = "{}{} && {}{}".format(
            base_cmd,
            "_default data_generation.simulated_device",
            base_cmd,
            "_default device.simulator_device_count",
        )
        print("Remove simulated device")
    else:
        """
        config_editor --conf application --filename ../conf/app_conf --set data_generation.simulated_device=1
        config_editor --conf application --filename ../conf/app_conf --set device.simulator_device_count=1
        """
        print("Add simulated device")
        cmd = "{}{} && {}{}".format(
            base_cmd,
            " data_generation.simulated_device=1",
            base_cmd,
            " device.simulator_device_count=1",
        )

    # print(cmd)

    proc = subprocess.Popen(
        cmd,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        stdin=subprocess.PIPE,
        shell=True,
        # Aliased by `text=True` in 3.7
        universal_newlines=True,
    )
    out, err = proc.communicate()

    if out:
        print(out, "#" * 30)
    if err:
        print(err, "#" * 30)

    print("Complete")



"""core.py
"""
import platform
import os


OS = platform.system()


def _minknow_path():
    return {
        "Darwin": os.path.join(os.sep, "Applications", "MinKNOW.app", "Contents", "Resources"),
        "Linux": os.path.join(os.sep, "opt", "ONT", "MinKNOW"),
        "Windows": os.path.join(os.sep, "Program Files", "OxfordNanopore", "MinKNOW"),
    }.get(OS, None)


def _python_path():
    return {
        "Darwin": os.path.join("ont-python", "bin", "python"),
        "Linux": os.path.join("ont-python", "bin", "python"),
        "Windows": os.path.join("ont-python", "python.exe"),
    }.get(OS, None)


def ont_python_path():
    return os.path.join(_minknow_path(), _python_path())


def bream_path():
    bream = os.path.join(
        "ont-python", "lib", "python2.7", "site-packages", "bream4", "configuration"
    )
    return os.path.join(_minknow_path(), bream)


def binaries_path():
    return os.path.join(_minknow_path(), "bin")


def full_config_path(filename):
    return os.path.join(bream_path(), filename)


def full_binaries_path(filename):
    return os.path.join(binaries_path(), filename)


def minknow_config_path(filename):
    return os.path.join(_minknow_path(), "conf", filename)



"""simulation.py
"""
from pathlib import Path
import toml
from fillet.core import full_config_path


def run(parser, args):
    args.toml = full_config_path(args.toml)
    print(args.toml)

    d = toml.load(args.toml)

    if args.bulkfile is not None:
        args.bulkfile = str(Path(args.bulkfile).expanduser())
        print("Adding simulation file")
        d["custom_settings"]["simulation"] = args.bulkfile
    else:
        print("Removing simulation parameter")
        d["custom_settings"].pop("simulation", None)

    with open(args.toml, "w") as fh:
        toml.dump(d, fh)

    print("File edited.")



"""main.py
"""
import traceback
from pathlib import Path
from argparse import ArgumentParser, HelpFormatter, SUPPRESS
from fillet.version import __version__
from fillet.core import bream_path


class NoneDefaultsHelpFormatter(HelpFormatter):
    """Help message formatter that adds default values to argument help string unless SUPPRESS or None"""

    def _get_help_string(self, action):
        help_str = action.help
        if action.default is not SUPPRESS and action.default is not None:
            help_str += " (default: " + str(action.default) + ")"
        return help_str


def run_command(parser, args):
    """..."""
    if args.command == "sim":
        import fillet.simulation as submodule
    elif args.command == "add":
        import fillet.add_device as submodule

    # run the chosen submodule.
    submodule.run(parser, args)


def main():
    """Execute the bulkvis command and pass args to sub-commands

    Returns
    -------
    None

    The available sub-commands:
    * sim    fillet.simulation
    * add    fillet.add_device
    """
    # TODO: Move arguments to each sub-command and build here (see serve)
    parser = ArgumentParser(
        prog="fillet",
        epilog="See '<command> --help' to read about a specific sub-command.",
    )
    parser.add_argument(
        "-v", "--version", action="version", version="fillet {}".format(__version__)
    )
    subparsers = parser.add_subparsers(dest="command", help="Sub-commands")

    """Create individual sub-commands"""

    """simulation"""
    sim_parser = subparsers.add_parser(
        "sim", help="Add or remove simulation parameters to an ONT config file"
    )
    sim_input = sim_parser.add_argument_group(title="Input sources")
    sim_input.add_argument(
        "-b",
        "--bulkfile",
        help="The bulk FAST file to use for the simulation, if not "
        "provided the simulation parameter will be removed from "
        "the toml file.",
        default=None,
    )
    toml_files = [
        f.name
        for f in Path(bream_path()).iterdir()
        if ".toml" in f.suffixes and "sequencing" in f.name
    ]
    sim_input.add_argument(
        "-t",
        help="The configuration file to add a simulation parameter to.",
        default=None,
        choices=toml_files,
        required=True,
        dest="toml",
    )
    sim_parser.set_defaults(func=run_command)

    """Add device"""
    add_parser = subparsers.add_parser("add", help="Add a simulation MinION to MinKNOW")
    add_parser.add_argument(
        "-r", help="Remove a simulated device.", action="store_true", dest="remove"
    )
    add_parser.set_defaults(func=run_command)

    """Parse arguments"""
    args = parser.parse_args()
    if args.command is not None:
        try:
            args.func(parser, args)
        except KeyboardInterrupt:
            print("Caught KeyboardInterrupt")
        except Exception as e:
            traceback.print_exc()
    else:
        parser.print_help()


if __name__ == "__main__":
    main()



from setuptools import setup, find_packages
from os import path


here = path.abspath(path.dirname(__file__))

with open(path.join(here, "README.md")) as fh:
    long_description = fh.read()

exec(open("fillet/version.py").read())

setup(
    name="fillet",
    version=__version__,
    author="Alexander Payne",
    author_email="alexander.payne@nottingham.ac.uk",
    description="Assistance scripts for ONT MinKNOW",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/alexomics/fillet",
    packages=find_packages(),
    python_requires=">=3.5",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    entry_points={
        "console_scripts": [
            "fillet=fillet.main:main",
        ],
    },
    package_dir={"fillet": "fillet"},
    install_requires=["toml",]  # TODO: give version numbers
)
